@@ -1,7 +1,7 @@
 /*
   curs_func.c
 
-  Copyright 2009, 2010, 2011, 2014 Free Software Foundation, Inc.
+  Copyright 2009, 2010, 2011, 2014, 2016 Free Software Foundation, Inc.
 
   This file is part of GNU Guile-Ncurses.
 
@@ -463,9 +463,6 @@ gucu_delay_output (SCM ms)
 SCM
 gucu_delscreen (SCM scr)
 {
-  SCREEN *c_scr;
-  FILE *c_ifp, *c_ofp;
-
   if (!isendwin ())
     /* terminal was freed while still in curses mode */
     return SCM_BOOL_F;
@@ -493,7 +490,7 @@ gucu_derwin (SCM orig, SCM nlines, SCM ncols, SCM begin_y, SCM begin_x)
   if (ret == 0)
     return SCM_BOOL_F;
 
-  return _scm_from_window (ret);
+  return _scm_from_window_full (orig, SCM_BOOL_F, ret);
 }
 
 /* Send to the physical terminal all the changes committed by calls to
@@ -1676,7 +1673,7 @@ gucu_subpad (SCM orig, SCM nlines, SCM ncols, SCM begin_y, SCM begin_x)
   if (ret == NULL)
     return SCM_BOOL_F;
 
-  return _scm_from_window (ret);
+  return _scm_from_window_full (orig, SCM_BOOL_F, ret);
 }
 
 /* Return a pointer to a new window that is within and shares memory with
@@ -1697,7 +1694,7 @@ gucu_subwin (SCM orig, SCM nlines, SCM ncols, SCM begin_y, SCM begin_x)
   if (ret == NULL)
     return SCM_BOOL_F;
 
-  return _scm_from_window (ret);
+  return _scm_from_window_full (orig, SCM_BOOL_F, ret);
 }
 
 /* When called with true, wsyncup is called automatically whenever the
@@ -1808,7 +1805,6 @@ gucu_unctrl (SCM ch)
 #else
   {
     chtype c_ch = _scm_xchar_to_chtype (ch);
-    char *c_name;
     return scm_from_locale_string (unctrl (c_ch));
   }
 #endif
@@ -2268,7 +2264,7 @@ gucu_wsyncup (SCM win)
 {
   WINDOW *c_win;
 
-  SCM_ASSERT (_scm_is_window (win), win, SCM_ARG1, "wsyncup");
+  SCM_ASSERT (_scm_is_window (win), win, SCM_ARG1, "%wsyncup");
 
   c_win = _scm_to_window (win);
 
@@ -2283,7 +2279,7 @@ gucu_wsyncdown (SCM win)
 {
   WINDOW *c_win;
 
-  SCM_ASSERT (_scm_is_window (win), win, SCM_ARG1, "wsyncdown");
+  SCM_ASSERT (_scm_is_window (win), win, SCM_ARG1, "%wsyncdown");
 
   c_win = _scm_to_window (win);
 
@@ -2507,8 +2503,8 @@ gucu_init_function ()
   scm_c_define_gsubr ("%winsnstr", 3, 0, 0, gucu_winsnstr);
   scm_c_define_gsubr ("%wmove", 3, 0, 0, gucu_wmove);
   scm_c_define_gsubr ("%wredrawln", 3, 0, 0, gucu_wredrawln);
-  scm_c_define_gsubr ("wsyncup", 1, 0, 0, gucu_wsyncup);
-  scm_c_define_gsubr ("wsyncdown", 1, 0, 0, gucu_wsyncdown);
+  scm_c_define_gsubr ("%wsyncup", 1, 0, 0, gucu_wsyncup);
+  scm_c_define_gsubr ("%wsyncdown", 1, 0, 0, gucu_wsyncdown);
   scm_c_define_gsubr ("%wtouchln", 4, 0, 0, gucu_wtouchln);
   scm_c_define_gsubr ("%wvline", 3, 0, 0, gucu_wvline);
 }
