@@ -1,7 +1,7 @@
 /*
 curs_spec.c
 
-Copyright 2009, 2010, 2011, 2014 Free Software Foundation, Inc.
+Copyright 2009, 2010, 2011, 2014, 2016 Free Software Foundation, Inc.
 
 This file is part of GNU Guile-Ncurses.
 
@@ -110,19 +110,8 @@ gucu_color_content (SCM s_color)
 SCM
 gucu_delwin (SCM win)
 {
-  WINDOW *c_win = _scm_to_window (win);
-
-  SCM_SET_SMOB_DATA (win, NULL);
-
-  if (c_win != NULL)
-    {
-      int ret = delwin (c_win);
-      if (ret == ERR)
-	return SCM_BOOL_F;
-      else
-	return SCM_BOOL_T;
-    }
-  return SCM_BOOL_F;
+  free_window (win);
+  return SCM_BOOL_T;
 }
 
 
@@ -320,7 +309,7 @@ gucu_winchnstr (SCM win, SCM n)
   {
     int ret;
     chtype *c_chstr = (chtype *) scm_malloc (sizeof (chtype) * (c_n + 1));
-    ret = winchnstr (_scm_to_window (win), c_chstr, scm_to_int (n));
+    ret = winchnstr (c_win, c_chstr, scm_to_int (n));
     if (ret != ERR)
       {
 	s_str = _scm_xstring_from_chstring (c_chstr);
