@@ -278,14 +278,23 @@ gucu_form_driver (SCM form, SCM c)
   SCM_ASSERT (SCM_CHARP (c) || scm_is_integer (c), c, SCM_ARG2,
 	      "form-driver");
 
-  int c_c;
   FORM *c_form = _scm_to_form (form);
+  int ret;
+
+#if (HAVE_NCURSESW == 1) && (HAVE_FORM_DRIVER_W)
+  if (SCM_CHARP (c))
+    ret = form_driver_w (c_form, OK, _scm_schar_to_wchar (c));
+  else
+    ret = form_driver_w (c_form, KEY_CODE_YES, (wchar_t) scm_to_int (c));
+#else
+  int c_c;
   if (SCM_CHARP (c))
     c_c = (unsigned char) _scm_schar_to_char (c);
   else
     c_c = scm_to_int (c);
 
-  int ret = form_driver (c_form, c_c);
+  ret = form_driver (c_form, c_c);
+#endif
 
   if (ret == E_BAD_ARGUMENT)
     scm_out_of_range ("form-driver", c);
